import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:foo_apple_pay/foo_apple_pay.dart';
import 'package:foo_apple_pay/apple_pay_event.dart';
import 'package:foo_apple_pay/pk_pass.dart';
import 'package:foo_apple_pay/pk_payment_pass.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    try {
      FooApplePay.setHostNameAndPath('SqHost', 'SqPath');

      bool supportsAppleWallet = await FooApplePay.deviceSupportsAppleWallet();
      print('$supportsAppleWallet');

      List<PKPass> localPasses = await FooApplePay.getLocalPasses();
      print('$localPasses');

      List<PKPaymentPass> remotePasses = await FooApplePay.getRemotePasses();
      print('$remotePasses');

      bool canAddCard = await FooApplePay.canAddCardWithId('1');
      print('$canAddCard');

      bool cardAlreadyAddedLocal =
          await FooApplePay.cardAlreadyAddedToLocalWallet('1');
      print('$cardAlreadyAddedLocal');

      bool cardAlreadyAddedRemote =
          await FooApplePay.cardAlreadyAddedToRemoteWallet('1');
      print('$cardAlreadyAddedRemote');

      FooApplePay.applePayCallback = testCallBack;

      FooApplePay.addCard('1', '2', '3', '4', '6');
      FooApplePay.addCard('1', '2', '3', '4', '6', '7', '8');

      platformVersion = 'Called all methods';
    } catch (e) {
      platformVersion = 'Failed due to $e';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Text('Running on: $_platformVersion\n'),
        ),
      ),
    );
  }
}

void testCallBack(ApplePayEvent event) {
  if (event.errorMessage != null) {
    print("Error message: " + event.errorMessage);
  } else {
    print("Error message is empty");
  }

  if (event.pass != null) {
    print("PKPaymentPass: " + event.pass.description);
  } else {
    print("PKPaymentPass is Empty");
  }

  if (event.error != null) {
    print("Error: " + event.error.toString());
  } else {
    print("Error is Empty");
  }
}
